"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = start;

require("universal-dotenv");

var _path = _interopRequireDefault(require("path"));

var _fs = require("fs");

var _os = require("os");

var _listr = _interopRequireDefault(require("listr"));

var _clearConsole = _interopRequireDefault(require("react-dev-utils/clearConsole"));

var _execa = _interopRequireDefault(require("execa"));

var _ngrok = require("ngrok");

var _babelPluginUniversalDotenv = require("babel-plugin-universal-dotenv");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// eslint-disable-line import/no-extraneous-dependencies
async function start({
  web = true,
  node = true,
  mongod = false,
  redis = false,
  ngrok = false
} = {}) {
  const {
    NODE_ENV = 'development'
  } = process.env;
  const env = { ...(0, _babelPluginUniversalDotenv.expand)(NODE_ENV),
    ...process.env
  };

  if (NODE_ENV === 'production') {
    return (0, _execa.default)('forever', [(0, _fs.existsSync)('./lib/index.node.js') ? 'lib/index.node.js' : 'lib/index.js'], {
      env: { ...env,
        NODE_ENV: 'production'
      },
      stdio: 'inherit'
    });
  }

  const {
    PORT = 3000,
    NODE_PORT = 4000,
    MONGOD_PORT = 27017,
    REDIS_PORT = 6379,
    BROWSER: PREVIOUS_BROWSER
  } = env;
  await new _listr.default([{
    title: 'Downloading mongod',
    enabled: () => mongod && !(0, _fs.existsSync)(_path.default.resolve((0, _os.homedir)(), '.mongodb-prebuilt')),
    task: () => (0, _execa.default)('mongod', ['--version']).stdout
  }, {
    title: 'Downloading redis',
    enabled: () => redis && !(0, _fs.existsSync)(_path.default.resolve((0, _os.homedir)(), '.redis-prebuilt')),
    task: () => (0, _execa.default)('redis-server', ['--version']).stdout
  }], {
    renderer: NODE_ENV === 'test' ? 'silent' :
    /* istanbul ignore next */
    'default',
    exitOnError: false,
    concurrent: true
  }).run();

  if (process.stdout.isTTY) {
    (0, _clearConsole.default)();
  }
  /*
   * https://github.com/strongloop/node-foreman/blob/master/lib/colors.js#L7
   * The order dictates the color
   */


  const formation = [['node', node && Number(node)], // magenta
  ['foo2', false], // blue
  ['web', web && Number(web)], // cyan
  ['mongod', mongod && Number(mongod)], // green
  ['foo5', false], // yellow
  ['redis', redis && Number(redis)] // red
  ];

  while (formation.length && !formation[formation.length - 1][1]) {
    formation.pop();
  }

  const ports = {
    web: PORT,
    node: NODE_PORT,
    mongod: MONGOD_PORT,
    redis: REDIS_PORT
  };
  return (0, _execa.default)('nf', ['start', '--procfile', _path.default.resolve(__dirname, 'Procfile'), '-x', formation.map(([key]) => ports[key] || 0).join(','), formation.map(([key, val], index) => val ? `${key}=${val}` : `foo${index + 1}=0`).join(',')], {
    env: { ...env,
      NODE_ENV: 'development',
      BROWSER: _path.default.resolve(__dirname, 'open-url.js'),
      PREVIOUS_BROWSER,
      HTTPS: false,
      ...(web && {
        PUBLIC_URL: `http://localhost:${PORT}`,
        SKIP_PREFLIGHT_CHECK: true
      }),
      ...(node && {
        REACT_APP_BACKEND_URL: `http://localhost:${NODE_PORT}`,
        src: (0, _fs.existsSync)('./src/index.node.js') ? 'src/index.node' : 'src',
        root: __dirname
      }),
      ...(mongod && {
        MONGODB_URI: `mongodb://localhost:${MONGOD_PORT}/database`,
        MONGOHQ_URL: `mongodb://localhost:${MONGOD_PORT}/database`,
        ORMONGO_URL: `mongodb://localhost:${MONGOD_PORT}/database`
      }),
      ...(redis && {
        OPENREDIS_URL: `redis://localhost:${REDIS_PORT}`,
        REDISCLOUD_URL: `redis://localhost:${REDIS_PORT}`,
        REDISGREEN_URL: `redis://localhost:${REDIS_PORT}`,
        REDISTOGO_URL: `redis://localhost:${REDIS_PORT}`,
        REDIS_URL: `redis://localhost:${REDIS_PORT}`
      }),
      ...(ngrok && Object.fromEntries((await Promise.all([web && ['PUBLIC_URL', {
        name: 'web',
        addr: PORT
      }], node && ['REACT_APP_BACKEND_URL', {
        name: 'node',
        addr: NODE_PORT
      }]].filter(Boolean).map(async ([key, opts]) => [key, await (0, _ngrok.connect)({
        bind_tls: true,
        // eslint-disable-line camelcase
        host_header: 'localhost',
        // eslint-disable-line camelcase
        ...opts
      })])))))
    },
    stdio: 'inherit'
  });
}
//# sourceMappingURL=react-node-scripts-start.js.map